//@target illustrator
//@targetengine main
//@strict on
//(c) illustratorScripts.com

(function () {


  function variate(h) {
    var doc = app.activeDocument;

    function getCurrentColors() {
      var doc = app.activeDocument;
      var currentColors = {};
      var fillRGB = getRGBFromColor(doc.defaultFillColor);

      var fillRGBColor = getRGBColorFromArray(fillRGB);
      currentColors.fill = fillRGBColor;
      var strokeRGB = getRGBFromColor(doc.defaultStrokeColor);
      var strokeRGBColor = getRGBColorFromArray(strokeRGB);
      currentColors.stroke = strokeRGBColor;
      return currentColors;
    }
    function getRGBFromColor(color) {
      
      if (color.typename == "RGBColor") {
        return [color.red, color.green, color.blue];
      } else if (color.typename == "GrayColor") {
        return [color.gray, color.gray, color.gray];
      } else if (color.typename == "CMYKColor") {
        var cyan = color.cyan / 100;
        var magenta = color.magenta / 100;
        var yellow = color.yellow / 100;
        var black = color.black / 100;
  
        var red = 1 - Math.min(1, cyan * (1 - black) + black);
        var green = 1 - Math.min(1, magenta * (1 - black) + black);
        var blue = 1 - Math.min(1, yellow * (1 - black) + black);
  
        return [Math.round(red * 255), Math.round(green * 255), Math.round(blue * 255)];
      } else {
        return [0, 0, 0];
      }
    }
    function getRGBColorFromArray(rgbArray) {
      var red = rgbArray[0];
      var green = rgbArray[1];
      var blue = rgbArray[2];

      red=range(red,0,255);
      green=range(green,0,255);
      blue=range(blue,0,255);

      var rgbColor = new RGBColor();
      rgbColor.red = red;
      rgbColor.green = green;
      rgbColor.blue = blue;
      return rgbColor;
    }
    function range(v,d,u){
      if(v>u) v=u;
      if(v<d) v=d;
      return v;
    }
    function rgbToHsl(color) {
      var r = color.red / 255;
      var g = color.green / 255;
      var b = color.blue / 255;
      var max = Math.max(r, g, b);
      var min = Math.min(r, g, b);
      var h, s, l = (max + min) / 2;
  
      if (max == min) {
        h = s = 0;
      } else {
        var d = max - min;
        s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
        switch (max) {
          case r:
            h = (g - b) / d + (g < b ? 6 : 0);
            break;
          case g:
            h = (b - r) / d + 2;
            break;
          case b:
            h = (r - g) / d + 4;
            break;
        }
        h /= 6;
      }
  
      return {
        hue: Math.round(h * 360),
        saturation: Math.round(s * 100),
        lightness: Math.round(l * 100)
      };
    }
    function hslToRgb(h, s, l) {
      var chroma = (1 - Math.abs((2 * l / 100) - 1)) * (s / 100);
      var hueSegment = h / 60;
      var x = chroma * (1 - Math.abs((hueSegment % 2) - 1));
      var r1, g1, b1;
      if (hueSegment >= 0 && hueSegment < 1) {
        r1 = chroma;
        g1 = x;
        b1 = 0;
      } else if (hueSegment >= 1 && hueSegment < 2) {
        r1 = x;
        g1 = chroma;
        b1 = 0;
      } else if (hueSegment >= 2 && hueSegment < 3) {
        r1 = 0;
        g1 = chroma;
        b1 = x;
      } else if (hueSegment >= 3 && hueSegment < 4) {
        r1 = 0;
        g1 = x;
        b1 = chroma;
      } else if (hueSegment >= 4 && hueSegment < 5) {
        r1 = x;
        g1 = 0;
        b1 = chroma;
      } else {
        r1 = chroma;
        g1 = 0;
        b1 = x;
      }
      var m = (l / 100) - (chroma / 2);
      var r = Math.round((r1 + m) * 255);
      var g = Math.round((g1 + m) * 255);
      var b = Math.round((b1 + m) * 255);
      return [r, g, b];
    }
  


    function pACC(pc, paths) {
      if (pc.constructor.name == "PathItem") {
        paths.push(pc)
      }
    }

    function iterateGroupsAndSubLayers(o, pathAndCompoundCheck, groupCheck, this_) {
      if (o.layers)
        for (var i = 0; i < o.layers.length; i++)
          iterateGroupsAndSubLayers(o.layers[i], pathAndCompoundCheck, groupCheck, this_);
      var items = o.constructor.name == "Array" ? o : o.pageItems;
      for (var i = 0; i < items.length; i++) {
        if (items[i].typename == "GroupItem") {
          iterateGroupsAndSubLayers(items[i], pathAndCompoundCheck, groupCheck, this_);
        } else {
          pathAndCompoundCheck && pathAndCompoundCheck(items[i], this_);
        }
      }
    }

    function variateOne(p,h,c){
      c=c||p.fillColor;
      c=getRGBFromColor(c);
      c=getRGBColorFromArray(c);
      c=rgbToHsl(c);
      p.filled=true;
      p.fillColor=getRGBColorFromArray(hslToRgb(c.hue+h[0]*Math.random()-h[0]/2,c.saturation+h[1]*Math.random()-h[1]/2,c.lightness+h[2]*Math.random()-h[2]/2));
    }

    var paths = [];
    iterateGroupsAndSubLayers(doc.selection, pACC, null, paths);
    

    for(var i=0;i<paths.length;i++){
      if(paths[i].filled) 
        variateOne(paths[i],h)
      
    }


    trace('OK');
  };



  function exec(func, params) {
    var bt = new BridgeTalk;
    bt.target = "illustrator";
    bt.body = func.toSource() + "(" + params.toSource() + ");"

    bt.send();
  }

  function tets(a) {
    alert(a)
  }

  function trace() {
    return
    var s = "";
    for (var i = 0; i < arguments.length; i++) {
      s += arguments[i] + " "
    }
    $.writeln(s);
  }





  var palette = new Window("palette", undefined, undefined, { borderless: true });
  palette.text = "HSL color randomizer";
  palette.preferredSize.width = 202;
  palette.preferredSize.height = 300;
  palette.orientation = "column";
  palette.alignChildren = ["left", "top"];
  palette.spacing = 10;
  palette.margins = 26;

  var statictext1 = palette.add("statictext", undefined, undefined, { name: "statictext1" });
  statictext1.text = "HUE variation 0..360";

  var edittext1 = palette.add('edittext {properties: {name: "edittext1"}}');
  edittext1.text = "20";
  edittext1.preferredSize.width = 100;

  var statictext2 = palette.add("statictext", undefined, undefined, { name: "statictext2" });
  statictext2.text = "SATURATION variation 0..100";
  statictext2.alignment = ["left", "top"];

  var edittext2 = palette.add('edittext {properties: {name: "edittext2"}}');
  edittext2.text = "10";
  edittext2.preferredSize.width = 100;

  var statictext3 = palette.add("statictext", undefined, undefined, { name: "statictext3" });
  statictext3.text = "LIGHTNESS variation 0..100";

  var edittext3 = palette.add('edittext {properties: {name: "edittext3"}}');
  edittext3.text = "10";
  edittext3.preferredSize.width = 100;

  //var divider1 = palette.add("panel", undefined, undefined, { name: "divider1" });
  //divider1.alignment = "fill";

  //var checkbox1 = palette.add("checkbox", undefined, undefined, { name: "checkbox1" });
  //checkbox1.text = "use fill and stroke colors";

  // GROUP1
  // ======
  var group1 = palette.add("group", undefined, { name: "group1" });
  group1.orientation = "row";
  group1.alignChildren = ["left", "center"];
  group1.spacing = 10;


  var button1 = group1.add("button", undefined, undefined, { name: "button1" });
  button1.text = "GO";

  var button2 = group1.add("button", undefined, undefined, { name: "button2" });
  button2.text = "close";
  button2.onClick = function () { palette.close() };


  button1.onClick = function () {
    exec(variate, [+edittext1.text,+edittext2.text,+edittext3.text]);
  };


  palette.show();


})();


